use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::DHL::API::Constants qw ( BUYONLINE_ATTRIBUTE_ID );
use DE_EPAGES::DHL::API::Tools;
use DE_EPAGES::DHL::DAL::DHLProducts qw ( DAL_GetDHLProducts );
use DE_EPAGES::DHL::API::Table::DHLProductList qw (GetAllPKeyDHLProductLists);
use Test::More;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            my $AreDHLProductsInstalled = scalar @{GetAllPKeyDHLProductLists()} > 0;
            is($AreDHLProductsInstalled, 1, 'Product imported - (on error please run DHL/Script/ImportDHLProductList.pl)');

            if($AreDHLProductsInstalled)
            {
                # tests
                TestGetProduct(
                    'PAK20.VERSI.DEU',
                    {
                        'Length'     => 60,
                        'Width'      => 30,
                        'Height'     => 15,
                        'LWH'        => 50,
                        'Weight'     => 1.5,
                        'Country'    => 'DEU',
                        'Attributes' => ['VERSI'],
                    },
                    { 'DHLProductID' => 'PAK20.VERSI.DEU', }
                );

                # tests
                TestGetProduct(
                    'PAK20.DEU',
                    {
                        'Length'     => 60,
                        'Width'      => 30,
                        'Height'     => 15,
                        'LWH'        => 50,
                        'Weight'     => 1.5,
                        'Country'    => 'DEU',
                        'Attributes' => ['Tracking'],
                    },
                    { 'DHLProductID' => 'PAK20.DEU', }
                );

                # tests
                TestGetProduct(
                    'PAK31.VERSI.DEU',
                    {
                        'Length'     => 120,
                        'Width'      => 60,
                        'Height'     => 60,
                        'LWH'        => undef,
                        'Weight'     => 31.5,
                        'Country'    => 'DEU',
                        'Attributes' => ['VERSI'],
                    },
                    {
                        'DHLProductID'          => 'PAK31.VERSI.DEU',
                        'OverallPriceAmountEUR' => '16.49',
                    }
                );
            }

            done_testing();
        }
    );
}

sub TestGetProduct {
    my ( $TestPrefix, $hInput, $hTestParams ) = @_;
    eval {
        my $ahDHLProducts = DAL_GetDHLProducts( @$hInput{qw(Length Width Height LWH Weight Country)}, BUYONLINE_ATTRIBUTE_ID );
        ## no critic (ProtectPrivateSubs)
        $ahDHLProducts = DE_EPAGES::DHL::API::Tools::_CheckValidityOfDHLProductIDs($ahDHLProducts);
        $ahDHLProducts = DE_EPAGES::DHL::API::Tools::_ExistsAttributesForDHLProduct( $ahDHLProducts, $hInput->{'Attributes'} );
        cmp_ok( scalar @$ahDHLProducts, '>', 0, $TestPrefix . ': ' . scalar @$ahDHLProducts . ' results.' );
        # check if one of the found DHLProducts has the correct values
        my $CorrectProductFound = 0;
        foreach my $hDHLProduct (@$ahDHLProducts) {
            my $IncorrectValue = 0;
            foreach ( keys %$hTestParams ) {
                $IncorrectValue = 1 unless $hDHLProduct->{$_} eq $hTestParams->{$_};
            }
            $CorrectProductFound = 1 unless $IncorrectValue;
        }
        is( $CorrectProductFound, 1, $TestPrefix);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        fail( $Error->longMessage );
    }
}

RunScript( Sub => \&Main );

1;
